#ifndef JOURNEYMODEL_H
#define JOURNEYMODEL_H

#include <QAbstractListModel>
#include <QStringList>
#include <QFile>

#include "journey.h"

class JourneyModel : public QAbstractListModel
{
    Q_OBJECT
public:
    enum JourneyRoles {
        StartRole = Qt::UserRole + 1,
        DurationRole,
        OvertookRole,
        OvertakenByRole
    };

    QHash<int, QByteArray> roleNames() const;

    JourneyModel(QObject *parent = 0);

    void addJourney(const Journey &journey);
    Q_INVOKABLE void addJourney(QDateTime start, quint32 duration, quint32 overtook, quint32 overtakenby);
    Q_INVOKABLE void editJourney(quint32 index, QDateTime start, quint32 duration, quint32 overtook, quint32 overtakenby);
    Q_INVOKABLE void deleteJourney(quint32 index);

    int rowCount(const QModelIndex & parent = QModelIndex()) const;

    QVariant data(const QModelIndex & index, int role = Qt::DisplayRole) const;

    void clear();

    void exportToFile(QFile & file);
    void importFromFile(QFile & file);

    Q_INVOKABLE static QDate epochToDate(quint64 epoch);
    Q_INVOKABLE static QTime epochToTime(quint64 epoch);
    Q_INVOKABLE static QDateTime epochToDateTime(quint64 epoch);

    QList<Journey> const & getData() const;

    void sort(int column, Qt::SortOrder order);
signals:
    // General signals
    void journeysChanged();

private:
    QHash<int, QByteArray> roles;
    QList<Journey> journeys;
};

#endif // JOURNEYMODEL_H
